unit Axes_Scale_Set_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TPoint = array [1..2] of real;
  TDraw2D = class(TForm)
    Image1: TImage;
    procedure Triangle(Sender: TObject);
    procedure Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Scale(x1,x2,y1,y2:Double);
    function  XCoor(x:Double):integer;
    function  YCoor(y:Double):integer;
    procedure PutPixel(X:TPoint;Color:DWord);
    procedure MoveTo(X:TPoint);
    procedure LineTo(X:TPoint;Color:DWord);
    procedure Line(X,Y:TPoint;Color:DWord);
    Procedure XAxis(x1,x2,y:Double;Color:DWord);
    Procedure YAxis(y1,y2,x:Double;Color:DWord);
    Procedure XCalib(x1,x2,y:Double;Color:DWord);
    procedure YCalib(y1,y2,x:Double;Color:DWord);
  end;

  var  Draw2D: TDraw2D;
       Red,Green,Blue:Byte;
       Unit_X,Unit_Y:Real;
       O,S          :TPoint;
       x1,x2,y1,y2  :Real;

implementation

{$R *.DFM}

  Procedure TDraw2D.Scale(x1,x2,y1,y2:Double);
  begin
     Unit_X:= Image1.Width/(x2-x1);
     Unit_Y:= Image1.Height/(y2-y1);
     O[1]  := -x1*Unit_X;
     O[2]  :=  y2*Unit_Y;
  end;

  function  TDraw2D.XCoor(x:Double):integer;
  begin
    XCoor:=trunc(O[1]+Unit_X*x);
  end;

  function  TDraw2D.YCoor(y:Double):integer;
  begin
     YCoor:=trunc(O[2]-Unit_Y*y);
  end;

  procedure TDraw2D.PutPixel(X:TPoint;Color:DWord);
  begin
     Image1.Canvas.Pixels[XCoor(X[1]),YCoor(X[2])]:=Color;
  end;

  procedure TDraw2D.MoveTo(X:TPoint);
  begin
    Image1.Canvas.MoveTo(XCoor(X[1]),YCoor(X[2]));
  end;

  procedure TDraw2D.LineTo(X:TPoint;Color:DWord);
  begin
    With Image1.Canvas do
     begin
       Pen.Color:=Color;
       LineTo(XCoor(X[1]),YCoor(X[2]))
     end;
  end;

  procedure TDraw2D.Line(X,Y:TPoint;Color:DWord);
  begin
     With Image1.Canvas do
     begin
       Pen.Color:=Color;
       MoveTo(XCoor(X[1]),YCoor(X[2]));
       LineTo(XCoor(Y[1]),YCoor(Y[2]));
     end;
  end;

  Procedure TDraw2D.XAxis(x1,x2,y:Double;Color:DWord);
  var A,B:TPoint;
  begin
    A[1]:=x1;A[2]:=y;B[1]:=x2;B[2]:=y;
    Line(A,B,Color);
  end;

  Procedure TDraw2D.YAxis(y1,y2,x:Double;Color:DWord);
  var A,B:TPoint;
  begin
    A[1]:=x;A[2]:=y1;B[1]:=x;B[2]:=y2;
    Line(A,B,Color);
  end;

procedure TDraw2D.XCalib(x1,x2,y:Double;Color:DWord);
  var AxisUnit        :Double;
      Text            :String;
      Lx,Exponent     :Integer;

   begin
   Image1.Canvas.Font.Color:=Color;
   AxisUnit:=1E30;Lx:=0;Exponent:=30;
   While (x2-x1)/AxisUnit<1 do
    begin
       AxisUnit:=AxisUnit/10;Exponent:=Exponent-1;
    end;
   Lx:=0;
   if x1<0 then
     Repeat Lx:=Lx-1 Until AxisUnit*(Lx-1)<x1
           else
     Repeat Lx:=Lx+1 Until AxisUnit*(Lx+1)>x1;
   With Image1.Canvas do
   repeat
      MoveTo(XCoor(AxisUnit*Lx),YCoor(y)-3);
      LineTo(XCoor(AxisUnit*Lx),YCoor(y)+3);
      Str(Lx,Text);
      TextOut(XCoor(AxisUnit*Lx)+2,YCoor(y)+5,Text);
      Lx:=Lx+1;
   until AxisUnit*(Lx)>x2;
   Str(Exponent,Text);
   Image1.Canvas.TextOut(XCoor(x2)+15,YCoor(y)+5,'*10');
   Image1.Canvas.TextOut(XCoor(x2)+35,YCoor(Y),Text);
   end;


procedure TDraw2D.YCalib(y1,y2,x:Double;Color:DWord);
  var AxisUnit        :Double;
      Text            :String;
      Ly,Exponent     :Integer;

   begin
   Image1.Canvas.Font.Color:=Color;
   AxisUnit:=1E30;Ly:=0;Exponent:=30;
   While (y2-y1)/AxisUnit<1 do
    begin
       AxisUnit:=AxisUnit/10;Exponent:=Exponent-1;
    end;
   Ly:=0;
   if y1<0 then
     Repeat Ly:=Ly-1 Until AxisUnit*(Ly-1)<y1
           else
     Repeat Ly:=Ly+1 Until AxisUnit*(Ly+1)>y1;
   With Image1.Canvas do
   repeat
      MoveTo(XCoor(x)-3,YCoor(AxisUnit*Ly));
      LineTo(XCoor(x)+3,YCoor(AxisUnit*Ly));
      Str(Ly,Text);
      TextOut(XCoor(x)-10,YCoor(AxisUnit*Ly)-15,Text);
      Ly:=Ly+1;
   until AxisUnit*(Ly)>y2;
   Str(Exponent,Text);
   Image1.Canvas.TextOut(XCoor(x)-30,YCoor(y2)-25,'*10');
   Image1.Canvas.TextOut(XCoor(x)-10,YCoor(y2)-30,Text);
   end;


procedure TDraw2D.Triangle(Sender: TObject);
var A,B,C:TPoint;
begin
    {Mazn}
    With Image1.Canvas do
    begin
       Brush.Color:=clWhite;Pen.Color:=clWhite;
       Rectangle(0,0,ClientWidth,ClientHeight);
    end;
    { Souradnice vrcholu }
    A[1]:=-20;A[2]:=-10;
    B[1]:= 10;B[2]:= 50;
    C[1]:= 50;C[2]:= 25;
    { Uzivatelska kreslici plocha }
    x1:=-40;x2:=70;y1:=-30;y2:=80;
    Scale(x1,x2,y1,y2);
    { Trojuhelnik }
    Image1.Canvas.Pen.Width:=2;
    Line(A,B,clBlue);Line(B,C,clBlue);Line(A,C,clBlue);
    {Popis trojuhelnika }
    With Image1.Canvas do
    begin
      Font.Color:=clBlue;Font.Height:=20;
      TextOut(XCoor(A[1])-5,YCoor(A[2])+10,'A');
      TextOut(XCoor(B[1]),YCoor(B[2])-25,'B');
      TextOut(XCoor(C[1])+5,YCoor(C[2]),'C');
    end;
    { Souradna soustava s popisem }
    With Image1.Canvas do
    begin
      Pen.Width:=1;{Font.Color:=Color};Font.Height:=10;
    end;
    XAxis(-35,65,0,clRed);YAxis(-25,75,0,clRed);
    XCalib(-30,50,0,clRed);YCalib(-20,60,0,clRed);
   
end;

procedure TDraw2D.Click(Sender: TObject);
begin
  Image1.Canvas.TextOut(100,100,'Click on me!');
end;

end.
